@@ -377,35 +377,18 @@ clutter_clone_new (ClutterActor *source)
   return g_object_new (CLUTTER_TYPE_CLONE, "source", source,  NULL);
 }
 
-static void
-clone_source_queue_redraw_cb (ClutterActor *source,
-			      ClutterActor *origin,
-			      ClutterClone *self)
-{
-  clutter_actor_queue_redraw (CLUTTER_ACTOR (self));
-}
-
-static void
-clone_source_queue_relayout_cb (ClutterActor *source,
-				ClutterClone *self)
-{
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
-}
-
 static void
 clutter_clone_set_source_internal (ClutterClone *self,
 				   ClutterActor *source)
 {
   ClutterClonePrivate *priv = self->priv;
 
+  if (priv->clone_source == source)
+    return;
+
   if (priv->clone_source != NULL)
     {
-      g_signal_handlers_disconnect_by_func (priv->clone_source,
-                                            G_CALLBACK (clone_source_queue_redraw_cb),
-					    self);
-      g_signal_handlers_disconnect_by_func (priv->clone_source,
-					    G_CALLBACK (clone_source_queue_relayout_cb),
-					    self);
+      _clutter_actor_detach_clone (priv->clone_source, CLUTTER_ACTOR (self));
       g_object_unref (priv->clone_source);
       priv->clone_source = NULL;
     }
@@ -413,10 +396,7 @@ clutter_clone_set_source_internal (ClutterClone *self,
   if (source != NULL)
     {
       priv->clone_source = g_object_ref (source);
-      g_signal_connect (priv->clone_source, "queue-redraw",
-			G_CALLBACK (clone_source_queue_redraw_cb), self);
-      g_signal_connect (priv->clone_source, "queue-relayout",
-			G_CALLBACK (clone_source_queue_relayout_cb), self);
+      _clutter_actor_attach_clone (priv->clone_source, CLUTTER_ACTOR (self));
     }
 
   g_object_notify_by_pspec (G_OBJECT (self), obj_props[PROP_SOURCE]);
